@@ -1,25 +1,28 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-import pytest
 import os
 import warnings
 
-from pandas import DataFrame, Series
-from pandas.compat import zip, iteritems
-from pandas.util._decorators import cache_readonly
-from pandas.core.dtypes.api import is_list_like
-import pandas.util.testing as tm
-from pandas.util.testing import (ensure_clean,
-                                 assert_is_valid_plot_return_object)
-import pandas.util._test_decorators as td
-
 import numpy as np
 from numpy import random
+import pytest
+
+from pandas.compat import iteritems, zip
+from pandas.util._decorators import cache_readonly
+import pandas.util._test_decorators as td
+
+from pandas.core.dtypes.api import is_list_like
+
+from pandas import DataFrame, Series
+import pandas.util.testing as tm
+from pandas.util.testing import (
+    assert_is_valid_plot_return_object, ensure_clean)
 
 import pandas.plotting as plotting
 from pandas.plotting._tools import _flatten
 
+
 """
 This is a common base class used for various plotting tests
 """
@@ -39,7 +42,7 @@ def _ok_for_gaussian_kde(kind):
         except ImportError:
             return False
 
-    return plotting._compat._mpl_ge_1_5_0()
+    return True
 
 
 @td.skip_if_no_mpl
@@ -50,30 +53,16 @@ class TestPlotBase(object):
         import matplotlib as mpl
         mpl.rcdefaults()
 
-        self.mpl_le_1_2_1 = plotting._compat._mpl_le_1_2_1()
-        self.mpl_ge_1_3_1 = plotting._compat._mpl_ge_1_3_1()
-        self.mpl_ge_1_4_0 = plotting._compat._mpl_ge_1_4_0()
-        self.mpl_ge_1_5_0 = plotting._compat._mpl_ge_1_5_0()
-        self.mpl_ge_2_0_0 = plotting._compat._mpl_ge_2_0_0()
         self.mpl_ge_2_0_1 = plotting._compat._mpl_ge_2_0_1()
+        self.mpl_ge_2_1_0 = plotting._compat._mpl_ge_2_1_0()
         self.mpl_ge_2_2_0 = plotting._compat._mpl_ge_2_2_0()
+        self.mpl_ge_2_2_2 = plotting._compat._mpl_ge_2_2_2()
+        self.mpl_ge_3_0_0 = plotting._compat._mpl_ge_3_0_0()
 
-        if self.mpl_ge_1_4_0:
-            self.bp_n_objects = 7
-        else:
-            self.bp_n_objects = 8
-        if self.mpl_ge_1_5_0:
-            # 1.5 added PolyCollections to legend handler
-            # so we have twice as many items.
-            self.polycollection_factor = 2
-        else:
-            self.polycollection_factor = 1
-
-        if self.mpl_ge_2_0_0:
-            self.default_figsize = (6.4, 4.8)
-        else:
-            self.default_figsize = (8.0, 6.0)
-        self.default_tick_position = 'left' if self.mpl_ge_2_0_0 else 'default'
+        self.bp_n_objects = 7
+        self.polycollection_factor = 2
+        self.default_figsize = (6.4, 4.8)
+        self.default_tick_position = 'left'
 
         n = 100
         with tm.RNGContext(42):
@@ -255,8 +244,8 @@ class TestPlotBase(object):
         else:
             labels = [t.get_text() for t in texts]
             assert len(labels) == len(expected)
-            for l, e in zip(labels, expected):
-                assert l == e
+            for label, e in zip(labels, expected):
+                assert label == e
 
     def _check_ticks_props(self, axes, xlabelsize=None, xrot=None,
                            ylabelsize=None, yrot=None):
@@ -461,7 +450,7 @@ class TestPlotBase(object):
                     assert isinstance(value.lines, dict)
                 elif return_type == 'dict':
                     line = value['medians'][0]
-                    axes = line.axes if self.mpl_ge_1_5_0 else line.get_axes()
+                    axes = line.axes
                     if check_ax_title:
                         assert axes.get_title() == key
                 else:
@@ -509,19 +498,11 @@ class TestPlotBase(object):
                 obj.plot(kind=kind, grid=True, **kws)
                 assert is_grid_on()
 
-    def _maybe_unpack_cycler(self, rcParams, field='color'):
+    def _unpack_cycler(self, rcParams, field='color'):
         """
-        Compat layer for MPL 1.5 change to color cycle
-
-        Before: plt.rcParams['axes.color_cycle'] -> ['b', 'g', 'r'...]
-        After : plt.rcParams['axes.prop_cycle'] -> cycler(...)
+        Auxiliary function for correctly unpacking cycler after MPL >= 1.5
         """
-        if self.mpl_ge_1_5_0:
-            cyl = rcParams['axes.prop_cycle']
-            colors = [v[field] for v in cyl]
-        else:
-            colors = rcParams['axes.color_cycle']
-        return colors
+        return [v[field] for v in rcParams['axes.prop_cycle']]
 
 
 def _check_plot_works(f, filterwarnings='always', **kwargs):
